#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#define JOKER    0x00
#define HEARTS   0x80
#define CLUBS    0x40
#define SPADES   0x20
#define DIAMONDS 0x10

#define ACE      0x01
#define TWO      0x02
#define THREE    0x03
#define FOUR     0x04
#define FIVE     0x05
#define SIX      0x06
#define SEVEN    0x07
#define EIGHT    0x08
#define NINE     0x09
#define TEN      0x0A

#define KING     0x0B
#define QUEEN    0x0C
#define JACK     0x0D

#define DIGIT    0x0F
#define SUIT     0xF0

#define RED      "\033[1;31m"
#define WHITE    "\033[1;37m"
#define RESET    "\033[0m"

struct deck {
  int card;
  struct deck *next;
};

struct deck *newCard(int card) {
  struct deck *deck = malloc(sizeof deck);
  deck->card = card;
  deck->next = NULL;
  return deck;
}

void appendCard(struct deck *dest, struct deck *source) {
  while(dest->next != NULL) {
    dest = dest->next;
  }
  dest->next = source;
}

char *cardString(struct deck *card) {
  char *string = malloc(10);
  int suit = card->card & SUIT;
  int digit = card->card & DIGIT;

  switch(suit) {
    case HEARTS:
      strcat(string, RED);
      strcat(string, "H");
      break;
    case DIAMONDS:
      strcat(string, RED);
      strcat(string, "D");
      break;
    case CLUBS:
      strcat(string, WHITE);
      strcat(string, "C");
      break;
    case SPADES:
      strcat(string, WHITE);
      strcat(string, "S");
      break;
    case JOKER:
      strcat(string, WHITE);
      strcat(string, "J");
      break;
  }

  if(digit == ACE)        strcat(string, "A");
  else if(digit == KING)  strcat(string, "K");
  else if(digit == QUEEN) strcat(string, "Q");
  else if(digit == JACK)  strcat(string, "J");
  else if(digit >= TWO && digit <= TEN) {
    char digits[2];
    sprintf(digits, "%d", digit);
    strcat(string, digits);
  }

  strcat(string, RESET);

  return string;
}

void printDeck(struct deck *deck) {
  char *string;
  while(deck != NULL) {
    string = cardString(deck);
    printf("%s\n", string);
    deck = deck->next;
  }
}

struct deck *gen52(void) {
  struct deck *deck = newCard(JOKER);
  struct deck *root = deck;
  for(int i = DIAMONDS; i <= CLUBS + 1; i <<= 1) {
    for(int j = ACE; j < JACK + 1; j++) {
      struct deck *buf = newCard(i|j);
      appendCard(deck, buf);
      deck = deck->next;
    }
  }
  return root;
}

struct deck *randomizeDeck(struct deck *deck) {
  struct deck *root = deck;
  time_t t;
  srand((unsigned)time(&t));

  for(int i = 0; i < 2000; i++) {
    for(;;) {
      if(deck->next == NULL) break;
      else if(deck->next->next == NULL) break;

      struct deck *one = deck;
      struct deck *two = deck->next;
      struct deck *three = deck->next->next;

      if(rand() % 2) {
        one->next = three;
        two->next = root;
        root = two;
      }

      deck = deck->next;
    }
    deck = root;
  }
  return root;
}

int main(void) {
  struct deck *fiftytwo = gen52();
  fiftytwo = randomizeDeck(fiftytwo);
  printDeck(fiftytwo);
}
